﻿using Infragistics.Controls.Charts;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Interactivity;
using System.Windows.Media;
using System.Windows.Shapes;

namespace ConditionalColoringSL
{
    public class ColumnSeriesColoringBehavior : Behavior<ColumnSeries>
    {
        private XamDataChart _chart;
        private ColumnSeries Series { get { return this.AssociatedObject; } }

        public string MinimumMemberPath
        {
            get { return (string)GetValue(MinimumMemberPathProperty); }
            set { SetValue(MinimumMemberPathProperty, value); }
        }

        // Using a DependencyProperty as the backing store for MinimumMemberPath.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty MinimumMemberPathProperty =
            DependencyProperty.Register("MinimumMemberPath", typeof(string), typeof(ColumnSeriesColoringBehavior), new PropertyMetadata(string.Empty));

        public string MaximumMemberPath
        {
            get { return (string)GetValue(MaximumMemberPathProperty); }
            set { SetValue(MaximumMemberPathProperty, value); }
        }

        // Using a DependencyProperty as the backing store for MaximumMemberPath.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty MaximumMemberPathProperty =
            DependencyProperty.Register("MaximumMemberPath", typeof(string), typeof(ColumnSeriesColoringBehavior), new PropertyMetadata(string.Empty));

        public string ResultMemberPath
        {
            get { return (string)GetValue(ResultMemberPathProperty); }
            set { SetValue(ResultMemberPathProperty, value); }
        }

        // Using a DependencyProperty as the backing store for ResultMemberPath.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ResultMemberPathProperty =
            DependencyProperty.Register("ResultMemberPath", typeof(string), typeof(ColumnSeriesColoringBehavior), new PropertyMetadata(string.Empty));
        
        protected override void OnAttached()
        {
            Series.Loaded += _chart_Loaded;
        }

        void _chart_Loaded(object sender, RoutedEventArgs e)
        {
            _chart = this.Series.SeriesViewer as XamDataChart;
            if (_chart != null)
                _chart.RefreshCompleted += chart_RefreshCompleted;
        }

        protected override void OnDetaching()
        {
            if (_chart != null)
                _chart.RefreshCompleted -= chart_RefreshCompleted;
        }

        void chart_RefreshCompleted(object sender, EventArgs e)
        {
            if (!string.IsNullOrEmpty(MinimumMemberPath) && !string.IsNullOrEmpty(MaximumMemberPath) && !string.IsNullOrEmpty(ResultMemberPath) && this.Series.RootCanvas != null)
            {
                var rectangles = this.Series.RootCanvas.Children.Cast<FrameworkElement>().Where(x => x is Rectangle);
                foreach (var rect in rectangles)
                {
                    var dc = rect.DataContext as Infragistics.Controls.Charts.DataContext;
                    var minimum = (double)dc.Item.GetType().GetProperty(MinimumMemberPath).GetValue(dc.Item, null);
                    var maximum = (double)dc.Item.GetType().GetProperty(MaximumMemberPath).GetValue(dc.Item, null);
                    var result = (double)dc.Item.GetType().GetProperty(ResultMemberPath).GetValue(dc.Item, null);

                    if (result < minimum)
                        (rect as Rectangle).Fill = new SolidColorBrush(Colors.Green);
                    else if (result > maximum)
                        (rect as Rectangle).Fill = new SolidColorBrush(Colors.Red);
                    else
                        (rect as Rectangle).Fill = new SolidColorBrush(Colors.Yellow);
                }
            }
        }
    }
}
